package app.data.service;

import app.data.entity.Matchday;
import app.data.entity.Season;
import app.data.repository.MatchdayRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;
import org.vaadin.artur.helpers.CrudService;

import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class MatchdayService extends CrudService<Matchday, Integer> {

    private final MatchdayRepository repository;

    public MatchdayService(@Autowired MatchdayRepository repository) {
        this.repository = repository;
    }

    @Override
    protected MatchdayRepository getRepository() {
        return repository;
    }

    @NonNull
    public List<Matchday> getMatchdaysSorted(@NonNull Season season) {
        return repository.findAll().stream()
                .filter(matchday -> matchday.getSeason().equals(season))
                .sorted(Comparator.comparingInt(Matchday::getNumber))
                .collect(Collectors.toList());
    }

    public Optional<Matchday> getFirstMatchday(@NonNull Season season) {
        List<Matchday> matchdays = getMatchdaysSorted(season);
        if (matchdays.isEmpty()) {
            return Optional.empty();
        }
        return Optional.of(matchdays.get(0));
    }

    @NonNull
    public List<Matchday> getMatchdaysWithActivitySorted(@NonNull Season season) {
        return repository.findAll().stream()
                .filter(matchday -> matchday.getSeason().equals(season))
                .filter(this::hasActivity)
                .sorted(Comparator.comparingInt(Matchday::getNumber))
                .collect(Collectors.toList());
    }

    public Optional<Matchday> getLastMatchdayWithActivityOrElseFirstMatchday(@NonNull Season season) {
        List<Matchday> matchdaysWithActivity = getMatchdaysWithActivitySorted(season);
        if (matchdaysWithActivity.isEmpty()) {
            return getFirstMatchday(season);
        }
        return Optional.of(matchdaysWithActivity.get(matchdaysWithActivity.size()-1));
    }

    public boolean hasActivity(@NonNull Matchday matchday) {
        return matchday.getMatches().stream().mapToInt(match -> match.getGames().size()).sum() != 0;
    }
}
