package app.gameimage.service;

import app.data.chesscom.ChessComGame;
import app.data.entity.Game;
import app.data.service.ChessComService;
import app.gameimage.GameImageGenerator;
import app.gameimage.GameImageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Optional;

@Service
public class GameImageService {

    private final ChessComService chessComService;

    public GameImageService(@Autowired ChessComService chessComService) {
        this.chessComService = chessComService;
    }

    public Optional<String> getVaadinImagePath(Game game) {
        return getImagePath(game).map(path -> path.replace("src/main/resources/META-INF/resources/", ""));
    }

    public Optional<String> getImagePath(Game game) {
        String path = GameImageUtils.IMAGE_DEST_PATH + getGameNumber(game) + GameImageUtils.EXTENSION;
        File f = new File(path);
        if(f.isFile()) {
            return Optional.of(path);
        }
        Optional<String> optionalFen = getFen(game);

        if (optionalFen.isPresent()) {
            GameImageUtils.writeImage(GameImageGenerator.generateImage(optionalFen.get()), getGameNumber(game));
            return Optional.of(path);
        }
        return Optional.empty();
    }

    private String getGameNumber(Game game) {
        String[] url = game.getGameInfo().getUrl().split("/");
        return url[url.length -1];
    }

    private Optional<String> getFen(Game game) {
        return chessComService.getChessComGame(game).map(ChessComGame::getFen);
    }
}
